'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.askConfirmation = exports.registerCommand = exports.notify = exports.notifier = undefined;

var _updateNotifier = require('update-notifier');

var _updateNotifier2 = _interopRequireDefault(_updateNotifier);

var _prettyError = require('pretty-error');

var _prettyError2 = _interopRequireDefault(_prettyError);

var _caporal = require('caporal');

var _caporal2 = _interopRequireDefault(_caporal);

var _inquirer = require('inquirer');

var _inquirer2 = _interopRequireDefault(_inquirer);

var _pTry = require('p-try');

var _pTry2 = _interopRequireDefault(_pTry);

var _package = require('../../package.json');

var _package2 = _interopRequireDefault(_package);

var _initialized = require('../actions/initialized');

var _initialized2 = _interopRequireDefault(_initialized);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

const pe = new _prettyError2.default();

pe.appendStyle({
  'pretty-error > trace > item': {
    marginBottom: 0
  }
});

const notifier = exports.notifier = (0, _updateNotifier2.default)({ pkg: _package2.default });

const notify = exports.notify = () => notifier.notify();

const registerCommand = exports.registerCommand = ({ name, alias, description, initialization, action, args = [], opts = [] }) => {
  const command = _caporal2.default.command(name, description);

  if (alias) {
    command.alias(alias);
  }

  args.forEach(argument => {
    command.argument(...argument);
  });

  opts.forEach(option => {
    command.option(...option);
  });

  command.action((args, options, logger) => {
    if (initialization && !(0, _initialized2.default)()) {
      logger.info('Cloud link not initialized! run `cloud-link init` first');

      return;
    }

    return (0, _pTry2.default)(() => action({ args, options, logger })).then(() => {
      notify();
    }).catch(err => {
      console.error(pe.render(err));
    });
  });
};

const askConfirmation = exports.askConfirmation = ({ shouldAsk = true, message, action }) => {
  if (!shouldAsk) {
    return action();
  }

  return _inquirer2.default.prompt([{
    type: 'confirm',
    name: 'confirm',
    message,
    default: false
  }]).then(({ confirm }) => confirm && action());
};
//# sourceMappingURL=util.js.map